import os
import unittest
from tempfile import TemporaryDirectory
from typing import Tuple, List

from parameterized import parameterized

from pdfconduit import Info, Upscale
from pdfconduit.utils.driver import Driver
from tests import *


def scaling_params() -> List[Tuple[str, Driver, float]]:
    return [
        ("{}_{}x".format(driver.name, str(round(scale * 100))), driver, scale)
        for driver in Driver
        for scale in [0.5, 1.5, 2.0, 3.0]
    ]


class TestUpscale(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.pdf_path = pdf_path
        cls.temp = TemporaryDirectory()

    @classmethod
    def tearDownClass(cls):
        cls.temp.cleanup()

    @parameterized.expand(scaling_params)
    def test_scale(self, name: str, driver: Driver, scale: float):
        scaled = (
            Upscale(
                pdf_path,
                scale=scale,
                suffix=name,
                tempdir=self.temp.name,
            )
            .use(driver)
            .upscale()
        )

        self.assertPdfExists(scaled)
        self.assertPdfScaled(scale, scaled)

    def assertPdfScaled(self, scale, scaled):
        self.assertEqual(
            Info(scaled).size, tuple([i * scale for i in Info(pdf_path).size])
        )
        self.assertEqual(Info(scaled).pages, Info(pdf_path).pages)

    def assertPdfExists(self, scaled):
        self.assertTrue(os.path.isfile(scaled))


if __name__ == "__main__":
    unittest.main()



import os
import unittest
from tempfile import TemporaryDirectory
from typing import List

from parameterized import parameterized

from pdfconduit import Info, slicer
from tests import *


def slice_params() -> List[str]:
    return [
        "1to1",
        "4to7",
        "2to6",
        "1to8",
        "4to9",
    ]


class TestSlice(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.pdf_path = pdf_path

    def setUp(self):
        self.temp = TemporaryDirectory()

    def tearDown(self):
        self.temp.cleanup()

    @parameterized.expand(slice_params)
    def test_slice(self, slice_range: str):
        fp, lp = tuple(slice_range.split("to"))
        fp = int(fp)
        lp = int(lp)

        sliced = slicer(
            self.pdf_path,
            first_page=fp,
            last_page=lp,
            tempdir=self.temp.name,
        )

        self.assertPdfExists(sliced)
        self.assertCorrectPagesSliced(fp, lp, sliced)

    def assertCorrectPagesSliced(self, fp, lp, sliced):
        self.assertEqual(Info(sliced).pages, len(range(fp, lp + 1)))

    def assertPdfExists(self, sliced):
        self.assertTrue(os.path.isfile(sliced))


if __name__ == "__main__":
    unittest.main()



import os
import unittest
from tempfile import TemporaryDirectory
from typing import Tuple, List

from parameterized import parameterized

from pdfconduit.conduit import Watermark
from pdfconduit.conduit.watermark.label import Label
from pdfconduit.utils import Info
from tests import *


def watermark_params() -> List[Tuple[str, str, bool, bool]]:
    return [
        # name, method, flatten, underneath
        ("pdfrw", "pdfrw", False, False),
        ("pdfrw_underneath", "pdfrw", False, True),
        ("pdfrw_overlay", "pdfrw", False, False),
        ("pdfrw_flattened", "pdfrw", True, False),
        ("pdfrw_flattened_underneath", "pdfrw", True, True),
        ("pypdf", "pypdf", False, False),
        ("pypdf_underneath", "pypdf", False, True),
        ("pypdf_overlay", "pypdf", False, False),
        ("pypdf_flattened", "pypdf", True, False),
        ("pypdf_flattened_underneath", "pypdf", True, True),
    ]


class TestWatermark(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.pdf_path = pdf_path

    def setUp(self):
        self.temp = TemporaryDirectory()
        self.address = "43 Indian Lane"
        self.town = "Franklin"
        self.state = "MA"
        self.rotate = 30
        self.owner_pw = "foo"
        self.user_pw = "baz"

    def tearDown(self):
        self.temp.cleanup()

    @parameterized.expand(watermark_params)
    def test_watermark(
        self, name: str, method: str, flatten: bool = False, underneath: bool = False
    ):
        watermarker = Watermark(
            self.pdf_path, use_receipt=False, open_file=False, tempdir=self.temp.name
        )
        watermark = watermarker.draw(
            text1=self.address,
            text2=str(self.town + ", " + self.state),
            opacity=0.08,
            rotate=self.rotate,
            flatten=flatten,
        )
        added = watermarker.add(
            self.pdf_path, watermark, method=method, underneath=underneath
        )

        self.assertPdfExists(watermark)
        self.assertPdfExists(added)
        self.assertPdfHasResources(added)

    def test_conduit_watermark_label(self):
        """Apply a watermark label to a PDF file."""
        label = os.path.basename(self.pdf_path)
        labeled = Label(
            self.pdf_path, label, tempdir=self.temp.name, suffix=None
        ).write(cleanup=False)

        self.assertPdfExists(labeled)
        self.assertPdfHasResources(labeled)

    def assertPdfExists(self, pdf):
        # Assert watermarked PDF file exists
        self.assertTrue(os.path.exists(pdf))

    def assertPdfHasResources(self, pdf):
        # Assert watermarked PDF has page resources
        self.assertTrue(Info(pdf).resources())


if __name__ == "__main__":
    unittest.main()



import os
import unittest

from pdfconduit import Info, Flatten
from tests import *


class TestFlatten(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.pdf_path = pdf_path
        cls.flat = None

    def tearDown(self):
        if os.path.exists(self.flat):
            os.remove(self.flat)

    def test_flatten(self):
        """Create a 'flattened' pdf file without layers."""
        self.flat = Flatten(self.pdf_path, suffix="flat").save()

        info_og = Info(self.pdf_path)
        info_flat = Info(self.flat)

        # Assert pdf file exists
        self.assertTrue(os.path.exists(self.flat))

        # Assert there are the same number of pages in the 'original' and 'flattened' pdf
        self.assertEqual(info_og.pages, info_flat.pages)

        # Confirm that PDF page sizes have not increased
        self.assertTrue(abs(info_og.size[0] / info_flat.size[0]) <= 1)
        self.assertTrue(abs(info_og.size[1] / info_flat.size[1]) <= 1)

        expected_equals_output(function_name_to_file_name(), self.flat)


if __name__ == "__main__":
    unittest.main()



import os
import unittest
from tempfile import NamedTemporaryFile
from typing import List, Tuple

from parameterized import parameterized

from pdfconduit.conduit import Encrypt
from pdfconduit.conduit.encrypt import Algorithms
from pdfconduit.utils import Info
from tests import *


def encryption_algo_params() -> List[Tuple[str, Algorithms, int]]:
    return [
        ("RC4-40", Algorithms.RC4_40, 2),
        ("RC4-128", Algorithms.RC4_128, 3),
        ("AES-128", Algorithms.AES_128, 4),
        ("AES-256", Algorithms.AES_256, 6),
        ("AES-256-R5", Algorithms.AES_256_r5, 5),
    ]


class TestEncrypt(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        # Encryption passwords
        cls.owner_pw = "foo"
        cls.user_pw = "baz"
        cls.pdf_path = pdf_path

    def setUp(self):
        self.temp = NamedTemporaryFile(suffix=".pdf", delete=False)

    def tearDown(self):
        if os.path.exists(self.temp.name):
            os.remove(self.temp.name)

    @parameterized.expand(encryption_algo_params)
    def test_encryption_algorithms(
        self, name: str, algorithm: Algorithms, expected_security_handler: int
    ):
        encrypted = Encrypt(
            self.pdf_path,
            self.user_pw,
            self.owner_pw,
            output=self.temp.name,
            bit128=algorithm.is_128bit,
            suffix=algorithm.name,
            algorithm=algorithm,
        )
        encrypted.encrypt()

        security = self._getPdfSecurity(encrypted)

        self.assertPdfExists(encrypted)
        self.assertEncrypted(encrypted)
        if algorithm.bit_length == 40:
            self.assert40BitEncryption(security)
        if algorithm.bit_length == 128:
            self.assert128BitEncryption(security, expected_security_handler)
        if algorithm.bit_length == 256:
            self.assert256BitEncryption(security, expected_security_handler)

        self.assertSecurityValue(security, 4)
        self.assertPermissions(encrypted, can_print=True)

    @parameterized.expand(encryption_algo_params)
    def test_permission_can_print(
        self, name: str, algorithm: Algorithms, expected_security_handler: int
    ):
        encrypted = Encrypt(
            self.pdf_path,
            self.user_pw,
            self.owner_pw,
            output=self.temp.name,
            bit128=algorithm.is_128bit,
            suffix=algorithm.name,
            algorithm=algorithm,
            allow_printing=True,
            allow_commenting=False,
        )
        encrypted.encrypt()

        self.assertPdfExists(encrypted)
        self.assertEncrypted(encrypted)

        self.assertPermissions(encrypted, can_print=True, can_modify=False)

    @parameterized.expand(encryption_algo_params)
    def test_permission_can_comment(
        self, name: str, algorithm: Algorithms, expected_security_handler: int
    ):
        encrypted = Encrypt(
            self.pdf_path,
            self.user_pw,
            self.owner_pw,
            output=self.temp.name,
            bit128=algorithm.is_128bit,
            suffix=algorithm.name,
            algorithm=algorithm,
            allow_printing=False,
            allow_commenting=True,
        )
        encrypted.encrypt()

        self.assertPdfExists(encrypted)
        self.assertEncrypted(encrypted)

        self.assertPermissions(encrypted, can_print=False, can_modify=True)

    @parameterized.expand(encryption_algo_params)
    def test_permission_can_print_and_comment(
        self, name: str, algorithm: Algorithms, expected_security_handler: int
    ):
        encrypted = Encrypt(
            self.pdf_path,
            self.user_pw,
            self.owner_pw,
            output=self.temp.name,
            bit128=algorithm.is_128bit,
            suffix=algorithm.name,
            algorithm=algorithm,
            allow_printing=True,
            allow_commenting=True,
        )
        encrypted.encrypt()

        self.assertPdfExists(encrypted)
        self.assertEncrypted(encrypted)

        self.assertPermissions(encrypted, can_print=True, can_modify=True)

    def test_password_byte_string(self):
        encrypted = Encrypt(
            self.pdf_path,
            self.user_pw,
            self.owner_pw,
            output=self.temp.name,
            suffix="byte_string",
            algorithm=Algorithms.RC4_128,
        )
        encrypted.encrypt()

        security = self._getPdfSecurity(encrypted)

        self.assertPdfExists(encrypted)
        self.assertEncrypted(encrypted)
        self.assert128BitEncryption(security, 3)
        self.assertSecurityValue(security, 4)

        self.assertPermissions(encrypted, can_print=True)

        # Assert user & owner password byte string is correct
        self.assertEqual(
            security["/O"],
            "ÐHÑRžS]˘—Í8V6{˘KJ’ß\x01æ7ÑnÊ\x06[¯Nd’\x0b",
        )

    def test_encrypted_pdf_has_metadata(self):
        encrypted = Encrypt(
            self.pdf_path,
            self.user_pw,
            self.owner_pw,
            output=self.temp.name,
            bit128=True,
            suffix="metadata",
        )
        encrypted.encrypt()

        self.assertPdfExists(encrypted)
        self.assertEncrypted(encrypted)

        metadata = Info(encrypted.output, self.user_pw).metadata
        self.assertEqual(metadata["/Producer"], "pdfconduit")
        self.assertEqual(metadata["/Creator"], "pdfconduit")
        self.assertEqual(metadata["/Author"], "Stephen Neal")

    def _getPdfSecurity(self, encrypted):
        return Info(encrypted.output, self.user_pw).security

    def assertPdfExists(self, encrypted):
        # Assert that pdf file exists
        self.assertTrue(os.path.exists(encrypted.output))

    def assertEncrypted(self, pdf):
        self.assertTrue(Info(pdf.output, self.user_pw).encrypted)

    def assert40BitEncryption(self, security):
        if "/Length" in security:
            self.assertEqual(security["/Length"], 40)

        # Assert standard security handler revision is 2
        self.assertEqual(security["/R"], 2)

    def assert128BitEncryption(self, security, security_handler_revision: int = 4):
        self.assertTrue("/Length" in security)
        self.assertIsInstance(security["/Length"], int)
        self.assertEqual(security["/Length"], 128)

        self.assertEqual(security["/Filter"], "/Standard")

        # Assert standard security handler revision is 3
        self.assertEqual(security["/R"], security_handler_revision)

        if security_handler_revision == 4:
            self.assertTrue("/CF" in security)
            self.assertEqual(security["/CF"]["/StdCF"]["/CFM"], "/AESV2")
        else:
            self.assertFalse("/CF" in security)

    def assert256BitEncryption(self, security, security_handler_revision: int = 5):
        self.assertTrue("/Length" in security)
        self.assertIsInstance(security["/Length"], int)
        self.assertEqual(security["/Length"], 256)

        self.assertEqual(security["/Filter"], "/Standard")

        # Assert standard security handler revision is 3
        self.assertEqual(security["/R"], security_handler_revision)
        self.assertEqual(security["/V"], 5)

        self.assertTrue("/CF" in security)
        self.assertEqual(security["/CF"]["/StdCF"]["/CFM"], "/AESV3")

    def assertSecurityValue(self, security, expected):
        self.assertTrue("/P" in security)
        self.assertEqual(security["/P"], expected)

    def assertPermissions(
        self,
        pdf,
        can_print=False,
        can_modify=False,
        can_copy=False,
        can_annotate=False,
        can_fill_forms=False,
        can_change_accessability=False,
        can_assemble=False,
        can_print_high_quality=False,
    ):
        permissions = Info(pdf.output, self.user_pw).permissions
        self.assertEqual(permissions.can_print(), can_print)
        self.assertEqual(permissions.can_modify(), can_modify)
        self.assertEqual(permissions.can_copy(), can_copy)
        self.assertEqual(permissions.can_annotate(), can_annotate)
        self.assertEqual(permissions.can_fill_forms(), can_fill_forms)
        self.assertEqual(
            permissions.can_change_accessability(), can_change_accessability
        )
        self.assertEqual(permissions.can_assemble(), can_assemble)
        self.assertEqual(permissions.can_print_high_quality(), can_print_high_quality)


if __name__ == "__main__":
    unittest.main()



import os
import unittest
from tempfile import TemporaryDirectory
from typing import List, Tuple

from parameterized import parameterized

from pdfconduit import Info, Rotate
from pdfconduit.utils.driver import Driver
from tests import *


def can_rotate_params() -> List[Tuple[str, Driver, int]]:
    return [
        ("pdfrw_90", Driver.pdfrw, 90),
        ("pdfrw_180", Driver.pdfrw, 180),
        ("pdfrw_270", Driver.pdfrw, 270),
        ("pypdf_90", Driver.pypdf, 90),
        ("pypdf_180", Driver.pypdf, 180),
        ("pypdf_270", Driver.pypdf, 270),
    ]


def cannot_rotate_params() -> List[Tuple[str, Driver, int]]:
    return [
        ("pypdf_135", Driver.pypdf, 135),
        ("pypdf_225", Driver.pypdf, 225),
        ("pypdf_315", Driver.pypdf, 315),
    ]


class TestRotate(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.pdf_path = pdf_path

    def setUp(self):
        self.temp = TemporaryDirectory()

    def tearDown(self):
        self.temp.cleanup()

    @parameterized.expand(can_rotate_params)
    def test_can_rotate(self, name: str, driver: Driver, rotation: int):
        rotated = (
            Rotate(
                self.pdf_path,
                rotation,
                suffix="rotated_{}_{}".format(driver.name, rotation),
                tempdir=self.temp.name,
            )
            .use(driver)
            .rotate()
        )

        self.assertPdfExists(rotated)
        self.assertPdfRotation(rotated, rotation)

    @parameterized.expand(cannot_rotate_params)
    def test_can_only_rotate_by_90_using_pypdf(
        self, name: str, driver: Driver, rotation: int
    ):
        with self.assertRaises(ValueError) as context:
            rotated = (
                Rotate(
                    self.pdf_path,
                    rotation,
                    suffix="rotated_{}_{}".format(driver.name, rotation),
                    tempdir=self.temp.name,
                )
                .use(driver)
                .rotate()
            )

        self.assertTrue(
            "Rotation angle must be a multiple of 90" in str(context.exception)
        )

    def assertPdfExists(self, pdf):
        self.assertTrue(os.path.isfile(pdf))

    def assertPdfDoesntExists(self, pdf):
        self.assertFalse(os.path.isfile(pdf))

    def assertPdfRotation(self, rotated, rotation):
        self.assertEqual(Info(rotated).rotate, rotation)


if __name__ == "__main__":
    unittest.main()



import os
import unittest
from tempfile import TemporaryDirectory
from typing import Tuple, List

from parameterized import parameterized

from pdfconduit import Info, Merge
from pdfconduit.utils.driver import Driver
from tests import *


def merge_params() -> List[Tuple[str, Driver]]:
    return [
        ("pdfrw", Driver.pdfrw),
        ("pypdf", Driver.pypdf),
    ]


class TestMerge(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        cls.pdfs = [
            os.path.join(test_data_dir, p)
            for p in ["article.pdf", "charts.pdf", "document.pdf", "manual.pdf"]
        ]

    def setUp(self):
        self.temp = TemporaryDirectory()

    def tearDown(self):
        self.temp.cleanup()

    @parameterized.expand(merge_params)
    def test_merge(self, name: str, driver: Driver):
        merger = Merge(
            self.pdfs, output_name="merged_{}".format(name), output_dir=self.temp.name
        )
        merger.use(driver).merge()

        # Assert merged file exists
        self.assertTrue(os.path.exists(merger.file))

        # Assert sum of pages in original pdf files equals sum of pages in merged pdf
        self.assertEqual(
            sum([Info(pdf).pages for pdf in self.pdfs]), Info(merger.file).pages
        )

        # Assert metadata was added correctly
        metadata = Info(merger.output).metadata
        if driver == Driver.pdfrw:
            self.assertEqual(metadata["/Producer"], "pdfconduit")
            self.assertEqual(metadata["/Creator"], "pdfconduit")
            self.assertEqual(metadata["/Author"], "Stephen Neal")
        else:
            self.assertEqual(metadata["/Producer"], "pypdf")


if __name__ == "__main__":
    unittest.main()
